* {
    outline-width: 0px;
    text-shadow: none;
}

.row-big {
    padding: 15px;
    border-radius: 3px;
	margin: 2px;
}

.row {
    padding: 8px 6px 8px 6px;
}

.devices-button {
    padding: 2px;
    min-width: 28px;
}

flowboxchild:selected  {
    background-color: transparent;
    color: @theme_fg_color;
}

.usagebar {
    border: none;
    background-color: transparent;
    background-image: none;
    padding-top: 0px;
    padding-bottom: 0px;
    margin-top: 0px;
    margin-bottom: 0px;
}

.usagebar progress {
    background-color: transparent;
    background-image: none;
    border: none;
    min-height: 6px;
}

.usagebar trough {
    border: 1px solid shade(@theme_bg_color, 0.9);
    background-color: transparent;
    background-image: none;
    min-height: 6px;
}

.usagebar-green progress {
    background-color: shade(green, 1.4);
}

.usagebar-orange progress {
    background-color: orange;
}

.usagebar-red progress {
    background-color: red;
}

.red {
    color: #e73435;
}

.progressbar-button {
    border: none;
    background-color: transparent;
    background-image: none;
    padding-top: 0px;
    padding-bottom: 0px;
    margin-top: 0px;
    margin-bottom: 0px;
}


.progressbar-button progress {
    background-color: @theme_selected_bg_color;
    background-image: none;
    border: none;
    border-radius: 1px;
    min-height: 2px;
}

.progressbar-button trough {
    border: 1px solid alpha(@theme_fg_color, 0.5);
    border-radius: 1px;
    min-height: 2px;
}

.typeahead {
    background-color: alpha(red, 0.1);
}

.typeahead.trackrow {
    background-color: alpha(red, 0.1);
}

.typeahead.albumrow {
    background-color: alpha(red, 0.1);
}

.typeahead.album-banner {
    background-color: alpha(red, 0.1);
}

.progress-bottom {
    border: none;
    border-radius: 0px;
    background-color: transparent;
    background-image: none;
    padding-top: 0px;
    padding-bottom: 0px;
    margin-top: 0px;
    margin-bottom: 0px;
}

.progress-bottom progress {
    background-color: @theme_selected_bg_color;
    background-image: none;
    border: none;
    border-radius: 0px;
    min-height: 6px;
}

.progress-bottom trough {
    border: none;
    border-radius: 0px;
    background-color: transparent;
    background-image: none;
    min-height: 6px;
}

.light-border {
    border: 1px solid shade(@theme_bg_color, 0.7);
    border-radius: 3px;
    padding: 10px;    
}

.light-background {
    background-color: shade(@theme_bg_color, 1.05);
    border: 1px solid shade(@theme_bg_color, 0.7);
    border-radius: 25%;
}

.no-top-border {
    border-top: none;
}

.background-opacity {
    background-color: @theme_bg_color;
    opacity: 0.99;
}

.selected {
    color: @theme_selected_bg_color;
}

.osd-popover {
    opacity: 0.99;
}

.no-border {
    background: transparent;
    background-image: none;
    border: none;
    box-shadow: none;
    color: alpha(@theme_fg_color, 0.8);
}

.no-border:hover {
    color: @theme_fg_color;
}

.monospace {
    font-family: Monospace;
}

.gnome-shell {
  font-family: Cantarell, Sans-Serif;
  font-size: 11pt;
}

.lyrics {
    font-weight: bold;
    min-width: 200px;
}

.text-xx-large {
    font-size: xx-large;
}

.text-x-large {
    font-size: x-large;
}

.text-large {
    font-size: large;
}

.text-medium {
    font-size: medium;
}

.text-small {
    font-size: smaller;
}

.mini-scale {
    padding: 1px;
}

.youtube-scale contents trough highlight {
    background-color: #e73435;  
    border-color: #e73435;
}

.borders {
    border: 1px solid shade(@theme_bg_color, 0.7);
    padding: 10px;
    border-radius: 4px;
}

.borders-left-top {
    border-top: 1px solid shade(@theme_bg_color, 0.7);
    border-left: 1px solid shade(@theme_bg_color, 0.7);
    border-top-left-radius: 10px;
}

.black {
    background-color: black;
    color: white;
}

.white {
    background-color: white;
    color: black;
}

.white-text {
    color: white;
}

.rounded {
    border-radius: 50%;
}

.up-btn {
    border: 1px solid shade(@theme_bg_color, 0.7);
    border-radius: 4px;
    border-bottom-left-radius: 0px;
    border-bottom-right-radius: 0px;
    min-height: 0px;
	min-width: 48px;
}

.cover-background {
    background-color: shade(@theme_bg_color, 1.5);
}

.cover-frame {
    border-radius: 2px;
    border: 1px solid alpha(black, 0.2);
    background-color: shade(@theme_bg_color, 1.5);
    padding: 3px;
    transition: box-shadow 250ms linear;
}

.cover-frame:selected {
    padding: 0px;
    border: 4px solid alpha(@theme_selected_bg_color, 0.5);
}

.cover-frame:visited {
    opacity: 0.9;
    box-shadow: 1px 1px 10px 4px alpha(@theme_selected_bg_color, 0.5);
}

.small-cover-frame {
    border-radius: 1px;
    border: 1px solid alpha(black, 0.2);
    background-color: shade(@theme_bg_color, 1.5);
    color: @theme_fg_color;
    transition: box-shadow 250ms linear;
}

.small-cover-frame:visited {
    opacity: 0.9;
    box-shadow: 1px 1px 10px 4px alpha(@theme_selected_bg_color, 0.5);
}

.transparent-cover-frame {
    border-radius: 2px;
    border: 1px solid alpha(black, 0.2);
    background-color: alpha(white, 0.5);
    padding: 2px;
}

.circle-icon {
    border: 1px solid alpha(black, 0.2);
    background-color: shade(@theme_bg_color, 1.5);
    color: alpha(@theme_fg_color, 0.8);
    border-radius: 50%;
}

.circle-icon-large {
    border: 1px solid alpha(black, 0.2);
    background-color: shade(@theme_bg_color, 1.5);
    color: alpha(@theme_fg_color, 0.8);
    border-radius: 50%;
    transition: box-shadow 250ms linear;
}

.circle-icon-large:visited {
    opacity: 0.9;
    box-shadow: 1px 1px 10px 4px alpha(@theme_selected_bg_color, 0.5);
}

.rounded-icon-large {
    border: 1px solid alpha(black, 0.2);
    background-color: shade(@theme_bg_color, 1.5);
    color: alpha(@theme_fg_color, 0.8);
    border-radius: 25%;
    transition: box-shadow 250ms linear;
}

.rounded-icon-large:visited {
    opacity: 0.9;
    box-shadow: 1px 1px 10px 4px alpha(@theme_selected_bg_color, 0.5);
}

.big-padding {
    padding: 10px;
}

.padding {
    padding: 5px;
}

.small-padding {
    padding: 1px;
}

.no-padding {
    padding: 0px;
    min-width: 0px;
    min-height: 0px;
}

.no-border {
    border: none;
    background-image: none;
}

.queued {
    border-radius: 5px;
    color: @theme_selected_fg_color;
    background-color: @theme_selected_bg_color;
}

.drag-down {
    background-image: -gtk-gradient (linear,
                                     left top,
                                     left bottom,
                                     from (@theme_bg_color),
                                     color-stop (0.9, @theme_bg_color),
                                     to (@theme_selected_bg_color));
}

.drag-up {
    background-image: -gtk-gradient (linear,
                                     left top,
                                     left bottom,
                                     from (@theme_selected_bg_color),
                                     color-stop (0.1, @theme_bg_color),
                                     to (@theme_bg_color));
}

.left-gradient {
    background-image: linear-gradient(0.25turn, transparent, alpha(black, 0.2) 20px, alpha(black, 0.4) 40px);
    background-color: transparent;
    opacity: 0;
}

.left-gradient:selected
{
    opacity: 1;
}

.trackswidget {
	background-color: transparent;
}

.albumrow {
    padding: 2px;
    border-radius: 5px;
    box-shadow: none;
	background-color: transparent;
  	color: inherit;
}

.albumrow-collapsed:hover {
	background-color: alpha(@theme_fg_color, 0.05);
  	color: inherit;
}

.albumrow .album-menu-button {
	border-color: transparent;
	background-color: transparent;
	color: transparent;
}

.albumrow:hover .album-menu-button {
  	color: inherit;	
}

.albumrow:selected {
    border-radius: 0px;
    background-color: @theme_selected_bg_color;
    color: @theme_selected_fg_color;
}

.albumrow:checked {
    color: @theme_fg_color;
    background-color: alpha(@theme_selected_bg_color, 0.1);
}
.albumrow:checked label {
    color: @theme_fg_color;
}

.albumrow:visited {
	background-color: alpha(@theme_fg_color, 0.05);
    color: @theme_fg_color;
}

.bold {
    font-weight: bold;
}

.adaptive-menu modelbutton.flat {
    min-height: 40px;
}

.adaptive-menu modelbutton.flat:hover {
    background-color: alpha(@theme_fg_color, 0.05);
}

.trackrow {
    padding: 2px;
    min-height: 40px;
    box-shadow: none;
	background-color: transparent;
  	color: inherit;
  	border: 1px solid transparent;
  	border-radius: 5px;
}

.trackrow:hover {
	background-color: alpha(@theme_fg_color, 0.05);
  	color: inherit;
}

.trackrow:selected {
    border-color: @theme_selected_bg_color;
}

.trackrow:visited {
	background-color: alpha(@theme_fg_color, 0.05);
    color: @theme_fg_color;
}

.default-banner {
    border-style: none;
    color: white;
    background-image: linear-gradient(60deg, shade(@theme_selected_bg_color, 0.4) 0%, shade(@theme_selected_bg_color, 0.5) 100%);

}

.album-banner {
    border: 1px solid alpha(black, 0.2);
    padding: 3px;
    border-radius: 4px;
    background-color: shade(@theme_bg_color, 0.9999);
    transition: box-shadow 250ms linear;
}

.album-banner:visited {
    opacity: 0.9;
    box-shadow: 1px 1px 10px 4px alpha(@theme_selected_bg_color, 0.5);
}

.album-banner:selected {
    padding-left: 0px;
    border-left: 4px solid alpha(@theme_selected_bg_color, 0.5);
}

.fullscreen-close-button {
    opacity: 0.8;
	box-shadow: none;
	margin: 5px;
	background-image: none;
	border-image-source: none;
    background-color: alpha(black, 0.5);
    color: white;
    border: 0px solid transparent;
    border-radius: 50%
}

.fullscreen-close-button:hover {
    color: inherit;
    background-color: alpha(black, 0.8);
  	opacity: 1;
}


.banner-button {
    opacity: 0.8;
	box-shadow: none;
	min-height: 34px;
	padding-left: 16px;
	padding-right: 16px;
	padding-top: 0px;
	padding-bottom: 0px;
	background-image: none;
	border-image-source: none;
    background-color: alpha(black, 0.5);
    color: white;
    border: 0px solid transparent;
}

.banner-button:disabled {
    opacity: 0.5;
}

.banner-button:checked {
    background-color: alpha(black, 0.7);
    opacity: 0.9;
    color: shade(@theme_selected_bg_color, 1.25);
}

.banner-button:visited {
    background-color: alpha(black, 0.7);
    opacity: 0.9;
}

.banner-button:hover {
    color: inherit;
    background-color: alpha(black, 0.8);
  	opacity: 1;
}

.banner-button:selected {
    color: @theme_selected_bg_color;
}

.banner-button-big {
    min-width: 42px;
	min-height: 36px;
	box-shadow: none;
	padding-left: 2px;
	padding-right: 2px;
	margin-top: 5px;
	margin-bottom: 5px;
	background-image: none;
	border-image-source: none;
    background-color: alpha(black, 0.5);
    color: white;
    border: 0px solid transparent;
}

.banner-button-big:disabled {
    opacity: 0.5;
}

.banner-button-big:hover {
    color: inherit;
    background-color: alpha(black, 0.8);
  	opacity: 1;
}

.menu-button {
    opacity: 0.8;
	box-shadow: none;
	min-height: 32px;
	padding-left: 10px;
	padding-right: 10px;
	padding-top: 0px;
	padding-bottom: 0px;
	background-image: none;
	border-image-source: none;
    border: 1px solid transparent;
    color: inherit;
}

.menu-button:disabled {
    opacity: 0.5;
}

.menu-button:checked {
    background-color: @theme_selected_bg_color;
}


.menu-button:hover {
    color: inherit;
    border-color: alpha(@theme_fg_color, 0.1);
  	opacity: 1;
}

.menu-button:selected {
    color: @theme_selected_bg_color;
}

.light-button {
    opacity: 0.8;
	box-shadow: none;
	min-height: 32px;
	padding-left: 10px;
	padding-right: 10px;
	padding-top: 0px;
	padding-bottom: 0px;
	background-image: none;
	border-image-source: none;
    border: 1px solid alpha(@theme_fg_color, 0.1);
    color: inherit;
}

.light-button:disabled {
    opacity: 0.5;
}

.light-button:checked {
    background-color: @theme_selected_bg_color;
    color: @theme_selected_fg_color;
}

.light-button:hover {
    border-color: alpha(@theme_fg_color, 0.2);
  	opacity: 1;
}

.image-reveal {
  -gtk-icon-transform: rotate(90deg);
}

@keyframes spin {
  to { -gtk-icon-transform: rotate(1turn); }
}

.image-rotate {
  animation-name: spin;
  animation-duration: 60s;
  animation-timing-function: linear;
  animation-iteration-count: infinite;
}

.image-rotate-fast {
  animation-name: spin;
  animation-duration: 2s;
  animation-timing-function: ease;
  animation-iteration-count: 1;
}

.opacity-transition {
    transition: opacity 1000ms linear;
    opacity: 0;
}

.opacity-transition:visited {
  opacity: 1;
}

.opacity-transition-fast {
    transition: opacity 200ms linear;
    opacity: 0;
}

.opacity-transition-fast:visited {
  opacity: 1;
}

.reveal-button-fullscreen {
    border-top-right-radius: 0px;
    border-bottom-right-radius: 0px;
}

@keyframes load {
  0% {
    background-color: alpha(@theme_selected_bg_color, 0);
  }
  50% {
    background-color: @theme_selected_bg_color;
  }
}

.load-animation {
  animation-name: load;
  animation-duration: 2s;
  animation-iteration-count: infinite;
}
